//==============================================================================
//
//   HashMapAttribute.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: HashMapAttribute.java,v 1.6 2003/09/11 15:21:54 holleis Exp $

package org.graffiti.attributes;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Maps a given id to an attribute. (e.g. if this Attribute contains 2 other
 * attributes, one StringAttribute with id &quot;Name&quot; and another
 * CollectionAttribute with id &quot;Color&quot; which contains three
 * IntegerAttributes ('red', 'green' and 'blue').  The map then contains only
 * the two key-value pairs 'Name'-StringAttribute and
 * 'Color'-CollectionAttribute. The subattributes 'red', 'green' and 'blue'
 * are not mapped in this Attribute!
 *
 * @version $Revision: 1.6 $
 *
 * @see CollectionAttribute
 * @see CompositeAttribute
 */
public class HashMapAttribute
    extends AbstractCollectionAttribute
    implements CollectionAttribute {
    //~ Constructors ===========================================================

    /**
     * Construct a new instance of a <code>HashMapAttribute</code>. The
     * internal HashMap is initialized empty.
     *
     * @param id the id of the attribute.
     */
    public HashMapAttribute(String id) {
        super(id);
        this.attributes = new HashMap();
    }

    //~ Methods ================================================================

    /**
     * Sets the collection of attributes contained within this
     * <tt>CollectionAttribute</tt> For each entry in the map, pre- and post-
     * AttributeAdded events are generated since method <code>add(Attribute
     * a)</code> is called for each attribute in the map.
     *
     * @param attrs the Map that contains all attributes.
     */
    public void setCollection(Map attrs) {
        assert attrs != null;
        attributes = new HashMap();

        Iterator it = attrs.values().iterator();

        if(getAttributable() == null) {
            while(it.hasNext()) {
                Attribute attr = (Attribute) it.next();
                this.add((Attribute) attr.copy(), false);
            }
        } else {
            while(it.hasNext()) {
                Attribute attr = (Attribute) it.next();
                this.add((Attribute) attr.copy());
            }
        }
    }

    /**
     * Returns a cloned map (shallow copy of map:  i.e.
     * <code>this.map.equals(getCollection())</code><b>but
     * not</b><code>this.map == getCollection()</code>)  between attributes'
     * ids and attributes contained in this attribute.
     *
     * @return a clone of the list of attributes in this attribute.3
     */
    public Map getCollection() {
        return (Map) ((HashMap) attributes).clone();
    }

    /**
     * Already done in constructor for this attribute type.
     *
     * @see org.graffiti.attributes.Attribute#setDefaultValue()
     */
    public void setDefaultValue() {
    }

    /**
     * Copies this <code>CollectionAttribute</code> and returns the copy. All
     * sub-attributes will be copied, too, i.e. a deep-copy is returned.
     *
     * @return a copy of the <code>CollectionAttribute</code>.
     */
    public Object copy() {
        HashMapAttribute copiedAttributes = new HashMapAttribute(this.getId());

        //M.S.: w�re es hier nicht sinnvoller �ber attributes.values() zu
        //      iterieren? getId() ist wahrscheinlich schneller als get(Id)
        //      bzw. hat eine kleinerer Konstante...
        for(Iterator i = attributes.keySet().iterator(); i.hasNext();) {
            String id = (String) i.next();
            Attribute attr = (Attribute) attributes.get(id);
            Attribute copiedAttribute = (Attribute) attr.copy();
            copiedAttribute.setParent(copiedAttributes);

            //	    copiedAttribute.setParent(this);
            copiedAttributes.attributes.put(id, copiedAttribute);
        }

        return copiedAttributes;
    }

    /**
     * Sets the value of the attribute by calling method
     * <code>setCollection(Map attrs)</code>. The "value" is the Collection of
     * attributes. For each entry in the map, pre- and post- AttributeAdded
     * events are generated.
     *
     * @param o the new value of the attribute.
     *
     * @exception IllegalArgumentException if the parameter has not the
     *            appropriate class for this attribute.
     */
    protected void doSetValue(Object o)
        throws IllegalArgumentException {
        assert o != null;

        HashMap attrs;

        try {
            attrs = (HashMap) o;
        } catch(ClassCastException cce) {
            throw new IllegalArgumentException("Wrong argument type " +
                "(Collection's value: HashMap - expected).");
        }

        setCollection(attrs);
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
