package org.graffiti.editor;

import org.graffiti.editor.actions.RunAlgorithm;
import org.graffiti.editor.dialog.DefaultParameterDialog;
import org.graffiti.editor.dialog.ParameterDialog;

import org.graffiti.graph.Graph;

import org.graffiti.managers.pluginmgr.DefaultPluginEntry;

import org.graffiti.plugin.algorithm.Algorithm;
import org.graffiti.plugin.algorithm.AlgorithmWithContextMenu;
import org.graffiti.plugin.algorithm.CalculatingAlgorithm;
import org.graffiti.plugin.algorithm.EditorAlgorithm;
import org.graffiti.plugin.parameter.Parameter;
import org.graffiti.plugin.view.MessageListener;

import org.graffiti.selection.Selection;
import org.graffiti.session.Session;

import java.beans.PropertyVetoException;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;

/**
 * Provides access to global variables, needed for various extensions to
 * Graffiti. Plugins can use the Preferences structure to save settings.
 */
public class GraffitiSingleton
{

    /**
     * DOCUMENT ME!
     */
    private static GraffitiSingleton instance;

    /**
     * DOCUMENT ME!
     */
    private volatile boolean plugins_MoveSelectionsAllowed = true;

    /**
     * DOCUMENT ME!
     */
    public volatile Object selectionSyncObject = new Object();

    /**
     * A list of frames, which are used by the pattern editor.
     */
    private List frames;
    
    private static LinkedList errorMessages=new LinkedList();

    /**
     * DOCUMENT ME!
     */
    private List patternSessions;

    /**
     * Stores the <code>MainFrame</code> object reference.
     */
    private MainFrame mainFrame;

    /**
     * Returns the single instance of this class. 
     *
     * @return The single instance of this "Singleton".
     */
    public static synchronized GraffitiSingleton getInstance()
    {
        if (instance == null) {
            instance = new GraffitiSingleton();
        }

        return instance;
    }

    /**
     * A global variable, for communication between the IPK Editing Tools
     * and some IPK Layouter. This will be later eventually be removed.
     *
     * @return If value is true, the selection should not be moved by the
     * layouter algorithms.
     */
    public synchronized boolean pluginSelectionMoveAllowed()
    {
        return plugins_MoveSelectionsAllowed;
    }

    /**
     * A global variable, for communication between the IPK Editing Tools
     * and some IPK Layouter. This will be later eventually be removed.
     *
     * @param value set to true, if the selection should not be moved by the
     * layouter algorithms.
     */
    public synchronized void pluginSetMoveAllowed(boolean value)
    {
        plugins_MoveSelectionsAllowed = value;
    }

    /**
     * Memorize the main frame.
     *
     * @param mainFrame The main frame.
     */
    public void setMainFrame(MainFrame mainFrame)
    {
        this.mainFrame = mainFrame;
    }

    /**
     * Returns a <code>Vector</code> which contains a list of sessions,
     * loaded in the main view.
     *
     * @return <code>Vector</code> with elements of the type
     *         <code>EditorSession</code>. Returns empty Vector, if no
     *         sessions are loaded.
     */
    public Vector getMainSessions()
    {
        Vector result = new Vector();

        Set sessions = getMainFrame()
                           .getSessions();

        for (Iterator it = sessions.iterator(); it.hasNext();) {
            Session curS = (Session) it.next();

            if ((patternSessions == null)
                    || (patternSessions.indexOf(curS) < 0)) {
                result.add(curS);
            }
        }

        return result;
    }

    /**
     * A <code>List</code> of the pattern sessions.
     * PatternSessions are sessions, which are loaded in the pattern tab.
     *
     * @return The pattern sessions.
     */
    public List getPatternSessionList()
    {
        return patternSessions;
    }

    /**
     * Returns the main frame (application window).
     *
     * @return The main frame.
     */
    public MainFrame getMainFrame()
    {
        return mainFrame;
    }

    /**
     * Returns a <code>Vector</code> which contains a list of graphs from the
     * main view.
     *
     * @return <code>Vector</code> with elements of the type
     *         <code>Graph</code>.
     */
    public Vector getMainGraphs()
    {
        Vector result = new Vector();

        for (Iterator it = mainFrame.getSessionsIterator(); it.hasNext();) {
            Session curS = (Session) it.next();

            if ((patternSessions == null)
                    || (patternSessions.indexOf(curS) < 0)) {
                result.add(curS.getGraph());
            }
        }

        return result;
    }

    /**
     * Returns a <code>Vector</code> which contains a list of pattern graphs.
     *
     * @return <code>Vector</code> with elements of the type
     *         <code>Graph</code>. If no patterns are loaded or available,
     *         this method returns an empty <code>Vector</code>.
     */
    public Vector getPatternGraphs()
    {
        Vector result = new Vector();
        if (patternSessions!=null) {
            for (int i = 0; i < patternSessions.size(); i++) {
                if (patternSessions.get(i) != null) {
                    result.add(((Session) patternSessions.get(i)).getGraph());
                }
            }
        }
        return result;
    }

    /**
     * Adds a new internal frame to the list of pattern editor frames.
     * Can be used by the method <code>isEditorFrameSelected</code> 
     * for the decission, whether a given frame is a editor frame or
     * a pattern editor frame.  
     *
     * @param frame New pattern editor frame.
     */
    public void addFrame(GraffitiInternalFrame frame)
    {
        if (frames == null) {
            frames = new ArrayList();
        }

        frames.add(frame);
    }

    /**
     * Adds a Session to the list of patternSessions. This method is called by
     * the patternInspector in the  action handler for the load and new
     * button action.
     *
     * @param session The new session, which should be known as a session,
     *        containing a pattern graph.
     */
    public void addPatternSession(Session session)
    {
        if (patternSessions == null) {
            patternSessions = new ArrayList();
        }

        patternSessions.add(session);
    }

    /**
     * Checks if an editor frame in the main view is selected.
     *
     * @return True, if an editor frame is selected.
     */
    public boolean isEditorFrameSelected()
    {
        boolean result = false;

        if (frames != null) {
            //MainFrame mf=getInstance().getMainFrame();
            //Session currentSession=mf.getActiveSession();
            for (Iterator it = frames.iterator(); it.hasNext();) {
                JInternalFrame frame = (JInternalFrame) it.next();

                // TODO check: a saved frame should never be null!
                if (frame != null) {
                    if (frame instanceof GraffitiInternalFrame) {
                        if (frame.isSelected()) {
                            result = true;
                            break;
                        }
                    }
                }
            }
        }

        return result;
    }

    /**
     * Unselects all editor frames in the main view.
     */
    public void framesDeselect()
    {
        if (frames != null) {
            //MainFrame mf=getInstance().getMainFrame();
            //Session currentSession=mf.getActiveSession();
            for (Iterator it = frames.iterator(); it.hasNext();) {
                JInternalFrame frame = (JInternalFrame) it.next();

                // TODO check: a saved frame should never be null!
                if (frame != null) {
                    try {
                        if (frame instanceof GraffitiInternalFrame) {
                            frame.setSelected(false);
                        }
                    } catch (PropertyVetoException e) {
                        // ignore, no problem
                    }
                }
            }
        }
    }

    /**
     * Returns a algorithm instance, defined by its name (e.g. menu item text)
     *
     * @param name The menu item text.
     *
     * @return The algorithm instance.
     */
    public Algorithm getAlgorithmInstanceFromFriendlyName(String name)
    {
        Collection plugins =
            GraffitiSingleton.getInstance()
                .getMainFrame()
                .getPluginManager()
                .getPluginEntries();

        for (Iterator pi = plugins.iterator(); pi.hasNext();) {
            DefaultPluginEntry curPluginEntry = (DefaultPluginEntry) pi.next();
            Algorithm[] myAlgos = curPluginEntry.getPlugin()
                                      .getAlgorithms();

            if (myAlgos.length > 0) {
                for (int i = 0; i < myAlgos.length; i++) {
                    if (myAlgos[i] instanceof AlgorithmWithContextMenu
                            && myAlgos[i] instanceof RunAlgorithm) {
                        AlgorithmWithContextMenu acm =
                            (AlgorithmWithContextMenu) myAlgos[i];

                        if (acm.getCurrentContextMenuItem()
                                .toString()
                                .equalsIgnoreCase(name)
                                || myAlgos[i].getName()
                                       .equalsIgnoreCase(name)) {
                            return myAlgos[i];
                        }
                    } 
                    Algorithm algo = myAlgos[i];

                    if (algo.getName()!=null && algo.getName()
                            .equalsIgnoreCase(name)) {
                        return myAlgos[i];
                    }
                }
            }
        }

        return null;
    }
    
    /**
     * Returns a plugin instance, given by its classname.
     * @param pluginDescription
     * @return The plugin instance, if the plugin is loaded.
     */
    public DefaultPluginEntry getPluginInstanceFromPluginDescription(String pluginDescription)
      {
          Collection plugins =
              GraffitiSingleton.getInstance()
                  .getMainFrame()
                  .getPluginManager()
                  .getPluginEntries();

          for (Iterator pi = plugins.iterator(); pi.hasNext();) {
              DefaultPluginEntry curPluginEntry = (DefaultPluginEntry) pi.next();
              if (curPluginEntry.getDescription().getName().toUpperCase().indexOf(pluginDescription.toUpperCase())>=0) {
                return curPluginEntry;
              }
          }

          return null;
      }    

    /**
     * Starts a plugin and returns, as soon as the plugin execution has
     * finished.
     *
     * @param pluginNameOrClassName of Algorithm to execute or Menu Item Text
     *        (from PluginMenu or Context Menu) or Classname of Plugin.
     * @param g Graph instance the plugin should work with.
     *
     * @return Empty String if success, otherwise Error message
     */
    public String runPlugin(final String pluginNameOrClassName, final Graph g)
    {
        
        class Exec {
            public void myRun(GraffitiSingleton gs) {
                Algorithm algo = gs.getAlgorithmInstanceFromFriendlyName(pluginNameOrClassName);
                
                gs.getMainFrame()
                .showMesssage("Execute plugin "
                        + pluginNameOrClassName + "",
                        MessageListener.INFO);
                runAlgorithm(algo);
            }
        }
        (new Exec()).myRun(this);
        
        /* 
        try {
            if (SwingUtilities.isEventDispatchThread()) {
                (new Exec()).myRun(this);
            } else {
                SwingUtilities.invokeAndWait(new Runnable() {
                        public void run()
                        {
                            (new Exec()).myRun(GraffitiSingleton.getInstance());
                        }
                    });
            }
        } catch (Exception e) {
            return e.getLocalizedMessage();
        } */

        return null;
    }
    
	public static void runAlgorithm(Algorithm algorithm) {
		Parameter[] parameters = algorithm.getParameters();
        ParameterDialog paramDialog = null;

        if((parameters != null) && (parameters.length != 0))
        {
            Selection activeSel = getInstance().getMainFrame().getActiveEditorSession()
                                           .getSelectionModel()
                                           .getActiveSelection();

            if(algorithm instanceof EditorAlgorithm)
            {
                paramDialog = ((EditorAlgorithm) algorithm).getParameterDialog(activeSel);
            }

            if(paramDialog == null)
            {
                paramDialog = new DefaultParameterDialog(getInstance().getMainFrame().getEditComponentManager(),
                		getInstance().getMainFrame(), parameters, activeSel, algorithm.getName());
            }

            // TODO load and save the preferences for this algorithm
            // TODO validate edited values
            if(!paramDialog.isOkSelected())
            {
                System.out.println("---alg not running!");

                return;
            }
        }
        if(parameters == null && algorithm instanceof EditorAlgorithm)
        {
            Selection activeSel = getInstance().getMainFrame().getActiveEditorSession()
                                           .getSelectionModel()
                                           .getActiveSelection();

            paramDialog = ((EditorAlgorithm) algorithm)
                .getParameterDialog(activeSel);
        }

        try
        {
            Parameter[] params = (paramDialog == null) ? new Parameter[] {  }
                                                       : paramDialog.getEditedParameters();
			algorithm.attach(getInstance().getMainFrame().getActiveSession().getGraph());
            algorithm.setParameters(params);

            algorithm.check();
            algorithm.execute();

            if(algorithm instanceof CalculatingAlgorithm)
            {
                JOptionPane.showMessageDialog(null,
                    "<html>Result of algorithm:<p>" +
                    ((CalculatingAlgorithm) algorithm).getResult().toString());
            }

            algorithm.reset();
            getInstance().getMainFrame().getActiveEditorSession().getSelectionModel()
                     .selectionChanged();
        }
        catch(Exception e)
        {
            e.printStackTrace();
            JOptionPane.showMessageDialog(null, 
            		e.getMessage(), 
					"Error while executing plugin", JOptionPane.ERROR_MESSAGE);
        }
	}


    /**
     * Write an error message to the console and add an entry to the logger.
     *
     * @param logger
     * @param errorMessage
     */
    public void reportError(Logger logger, String errorMessage)
    {
        System.err.println(errorMessage);
        logger.log(Level.SEVERE, errorMessage);
        // setStatusText(errorMessage, 3);
    }
    
    /**
     * Adds a errorMessage to a global list.
     * The error messages can be retreived with <code>getErrorMessages</code>
     * and cleared with <code>clearErrorMessages</code>.
     * @param errorMsg
     */
    public void addErrorMessage(String errorMsg) {
    	synchronized(errorMessages) {
    		errorMessages.add(errorMsg);
    	}
    }
    
    /**
     * Removes the current error messages. E.g. after showing them to the user.
     */
    public void clearErrorMessages() {
    	synchronized(errorMessages) {
    		errorMessages.clear();
    	}
    }

	/**
	 * Returns pending error messages that were not shown to the user 
	 * immediatly.
	 * @return Pending Error Messages
	 */
	public String[] getErrorMessages() {
		synchronized(errorMessages) {
			String[] result = new String[errorMessages.size()];
			int i=0;
			for (Iterator it=errorMessages.iterator(); it.hasNext(); ) {
				result[i++]=(String) it.next();
			}
			return result;
		}
	}
}


//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
