//==============================================================================
//
//   DefaultIOManager.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: DefaultIOManager.java,v 1.3 2003/11/03 12:37:17 forster Exp $

package org.graffiti.managers;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;

import javax.swing.JFileChooser;

import org.graffiti.core.GenericFileFilter;

import org.graffiti.managers.pluginmgr.PluginDescription;

import org.graffiti.plugin.GenericPlugin;
import org.graffiti.plugin.io.InputSerializer;
import org.graffiti.plugin.io.OutputSerializer;

/**
 * Handles the editor's IO serializers.
 *
 * @version $Revision: 1.3 $
 */
public class DefaultIOManager
    implements IOManager
{
    //~ Static fields/initializers =============================================

    /** The logger for the current class. */
    private static final Logger logger = Logger.getLogger(DefaultIOManager.class.getName());

    //~ Instance fields ========================================================

    /** The map from name of extension to its input serializer. */
    private HashMap inputSerializer;

    /** The map from name of extension to its output serializer. */
    private HashMap outputSerializer;

    /** The file chooser used to open and save graphs. */
    private JFileChooser fc;

    /** The list of listeners. */
    private List listeners;

    //~ Constructors ===========================================================

    /**
     * Constructs a new io manager.
     */
    public DefaultIOManager()
    {
        inputSerializer = new HashMap();
        outputSerializer = new HashMap();
        this.listeners = new LinkedList();
        fc = new JFileChooser();
    }

    //~ Methods ================================================================

    /*
     * @see org.graffiti.managers.IOManager#addInputSerializer(org.graffiti.plugin.io.InputSerializer)
     */
    public void addInputSerializer(InputSerializer is)
    {
        String[] inExtensions = is.getExtensions();

        for(int j = 0; j < inExtensions.length; j++)
        {
            inputSerializer.put(inExtensions[j], is.getClass());
            logger.info(inExtensions[j] + " added in the input" +
                " serializer map with the" + " associated class " +
                is.getClass().getName());
        }

        fireInputSerializerAdded(is);
    }

    /*
     * @see org.graffiti.managers.IOManager#addListener(org.graffiti.managers.IOManager.IOManagerListener)
     */
    public void addListener(IOManagerListener ioManagerListener)
    {
        listeners.add(ioManagerListener);
    }

    /*
     * @see org.graffiti.managers.IOManager#addOutputSerializer(org.graffiti.plugin.io.OutputSerializer)
     */
    public void addOutputSerializer(OutputSerializer os)
    {
        String[] outExtensions = os.getExtensions();

        for(int j = 0; j < outExtensions.length; j++)
        {
            outputSerializer.put(outExtensions[j], os.getClass());
            logger.info(outExtensions[j] + " added in the input" +
                " serializer map with the" + " associated class " +
                os.getClass().getName());
        }

        fireOutputSerializerAdded(os);
    }

    /*
     * @see org.graffiti.managers.IOManager#createInputSerializer(java.lang.String)
     */
    public InputSerializer createInputSerializer(String ext)
        throws IllegalAccessException, InstantiationException
    {
        for(Iterator it = inputSerializer.keySet().iterator(); it.hasNext();)
        {
            System.out.println("inputser ext: " + it.next());
        }

        for(Iterator it = outputSerializer.keySet().iterator(); it.hasNext();)
        {
            System.out.println("outputser ext: " + it.next());
        }

        Class isClass = (Class) inputSerializer.get(ext);

        return (InputSerializer) isClass.newInstance();
    }

    /*
     * @see org.graffiti.managers.IOManager#createOpenFileChooser()
     */
    public JFileChooser createOpenFileChooser()
    {
        fc.resetChoosableFileFilters();

        for(Iterator itr = inputSerializer.keySet().iterator(); itr.hasNext();)
        {
            fc.addChoosableFileFilter(new GenericFileFilter((String) itr.next()));
        }

        return fc;
    }

    /*
     * @see org.graffiti.managers.IOManager#createOutputSerializer(java.lang.String)
     */
    public OutputSerializer createOutputSerializer(String ext)
        throws IllegalAccessException, InstantiationException
    {
        Class osClass = (Class) outputSerializer.get(ext);

        return (OutputSerializer) osClass.newInstance();
    }

    /*
     * @see org.graffiti.managers.IOManager#createSaveFileChooser()
     */
    public JFileChooser createSaveFileChooser()
    {
        //        JFileChooser fc = new JFileChooser();
        fc.resetChoosableFileFilters();

        for(Iterator itr = outputSerializer.keySet().iterator(); itr.hasNext();)
        {
            fc.addChoosableFileFilter(new GenericFileFilter((String) itr.next()));
        }

        return fc;
    }

    /*
     * @see org.graffiti.managers.IOManager#hasInputSerializer()
     */
    public boolean hasInputSerializer()
    {
        return !inputSerializer.isEmpty();
    }

    /*
     * @see org.graffiti.managers.IOManager#hasOutputSerializer()
     */
    public boolean hasOutputSerializer()
    {
        return !outputSerializer.isEmpty();
    }

    /*
     * @see org.graffiti.managers.pluginmgr.PluginManagerListener#pluginAdded(org.graffiti.plugin.GenericPlugin, org.graffiti.managers.pluginmgr.PluginDescription)
     */
    public void pluginAdded(GenericPlugin plugin, PluginDescription desc)
    {
        // register add input serializers
        InputSerializer[] is = plugin.getInputSerializers();

        for(int i = 0; i < is.length; i++)
        {
            addInputSerializer(is[i]);
        }

        // register all output serializers
        OutputSerializer[] os = plugin.getOutputSerializers();

        for(int i = 0; i < os.length; i++)
        {
            addOutputSerializer(os[i]);
        }
    }

    /*
     * @see org.graffiti.managers.IOManager#removeListener(org.graffiti.managers.IOManager.IOManagerListener)
     */
    public boolean removeListener(IOManagerListener l)
    {
        return listeners.remove(l);
    }

    /**
     * Returns a string representation of the io manager.  Useful for
     * debugging.
     *
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return "inputSerializer: " + inputSerializer + " outputSerializer: " +
        outputSerializer;
    }

    /**
     * Informs every registered io manager listener about the addition of the
     * given input serializer.
     *
     * @param is the input serializer, which was added.
     */
    private void fireInputSerializerAdded(InputSerializer is)
    {
        for(Iterator i = listeners.iterator(); i.hasNext();)
        {
            IOManager.IOManagerListener l = (IOManager.IOManagerListener) i.next();

            l.inputSerializerAdded(is);
        }
    }

    /**
     * Informs every output serializer about the addition of the given output
     * serializer.
     *
     * @param os the output serializer, which was added.
     */
    private void fireOutputSerializerAdded(OutputSerializer os)
    {
        for(Iterator i = listeners.iterator(); i.hasNext();)
        {
            IOManager.IOManagerListener l = (IOManager.IOManagerListener) i.next();

            l.outputSerializerAdded(os);
        }
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
