//==============================================================================
//
//   SelectionModel.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: SelectionModel.java,v 1.4 2003/11/03 12:37:18 forster Exp $

package org.graffiti.selection;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Contains a list of selections and a reference to the current selection.
 *
 * @author flierl
 * @version $Revision: 1.4 $
 */
public class SelectionModel
{
    //~ Static fields/initializers =============================================

    /** DOCUMENT ME! */
    public static final String ACTIVE = "active";

    //~ Instance fields ========================================================

    /**
     * The list of selections.  Maps a <code>Selection.name</code> to a
     * <code>Selection</code> instance.
     */
    private Hashtable selections;

    /**
     * The list of listeners, that want to be informed about changes in the
     * selection model.
     *
     * @see SelectionListener
     */
    private List listeners;

    /** The current active selection. */
    private String activeSelection;

    //~ Constructors ===========================================================

    /**
     * Constructs a new <code>SelectionModel</code>.
     */
    public SelectionModel()
    {
        selections = new Hashtable();
        listeners = new LinkedList();
    }

    //~ Methods ================================================================

    /**
     * Sets the active selection to the given value. Informs all listeners
     * about the change.
     *
     * @param selectionName the name of the new selection.
     */
    public void setActiveSelection(String selectionName)
    {
        if(this.activeSelection != null)
        {
            Selection oldSel = (Selection) this.selections.get(this.activeSelection);
            oldSel.clear();
            this.selectionChanged();
        }

        this.activeSelection = selectionName;
        this.selectionChanged();
    }

    /**
     * Sets the active selection to the given value. Informs all listeners
     * about the change.
     *
     * @param sel the name of the new selection.
     */
    public void setActiveSelection(Selection sel)
    {
        if(this.activeSelection != null)
        {
            Selection oldSel = (Selection) this.selections.get(this.activeSelection);
            oldSel.clear();
            this.selectionChanged();
        }

        this.activeSelection = sel.getName();

        if(selections.put(sel.getName(), sel) == null)
        {
            SelectionEvent selectionEvent = new SelectionEvent(sel);
            selectionEvent.setAdded(true);

            for(Iterator it = listeners.iterator(); it.hasNext();)
            {
                ((SelectionListener) it.next()).selectionListChanged(selectionEvent);
            }
        }

        this.selectionChanged();
    }

    /**
     * Returns the active selection.
     *
     * @return DOCUMENT ME!
     */
    public Selection getActiveSelection()
    {
        if(activeSelection == null)
        {
            return null;
        }
        else
        {
            return (Selection) selections.get(activeSelection);
        }
    }

    //    /**
    //     * Creates a new empty selection and sets this as the new active selection.
    //     * Returns this new empty, active selection. Informs all
    //     * <code>SelectionListener</code>s.
    //     *
    //     * @return DOCUMENT ME!
    //     */
    //    public Selection setEmptySelection()
    //    {
    //        Selection newSel = new Selection(ACTIVE);
    //
    //        // record that all previously marked elements have now been unmarked
    //        Selection oldSel = (Selection) selections.get(activeSelection);
    //        Map newUnmarked = new HashMap();
    //
    //        for(Iterator it = oldSel.getNodes().iterator(); it.hasNext();)
    //        {
    //            newUnmarked.put(it.next(), null);
    //        }
    //
    //        for(Iterator it = oldSel.getEdges().iterator(); it.hasNext();)
    //        {
    //            newUnmarked.put(it.next(), null);
    //        }
    //
    //        newUnmarked.putAll(oldSel.getNewUnmarked());
    //        newSel.setNewUnmarked(newUnmarked);
    //        newSel.setNewMarked(oldSel.getNewMarked());
    //
    //        this.add(newSel);
    //        this.setActiveSelection(ACTIVE);
    //
    //        return this.getActiveSelection();
    //    }
    //
    //    /**
    //     * Creates a new empty selection and sets this as the new active selection.
    //     * Returns this new empty, active selection. Does not inform any
    //     * <code>SelectionListener</code>s.
    //     *
    //     * @return DOCUMENT ME!
    //     */
    //    public Selection setEmptySelectionNonNotify()
    //    {
    //        Selection newSel = new Selection(ACTIVE);
    //
    //        // record that all previously marked elements have now been unmarked
    //        Selection oldSel = (Selection) selections.get(activeSelection);
    //        Map newUnmarked = new HashMap();
    //
    //        for(Iterator it = oldSel.getNodes().iterator(); it.hasNext();)
    //        {
    //            newUnmarked.put(it.next(), null);
    //        }
    //
    //        for(Iterator it = oldSel.getEdges().iterator(); it.hasNext();)
    //        {
    //            newUnmarked.put(it.next(), null);
    //        }
    //
    //        newUnmarked.putAll(oldSel.getNewUnmarked());
    //        newSel.setNewUnmarked(newUnmarked);
    //        newSel.setNewMarked(oldSel.getNewMarked());
    //
    //        this.add(newSel);
    //        this.activeSelection = ACTIVE;
    //
    //        //		this.setActiveSelection(ACTIVE);
    //        return newSel;
    //    }

    /**
     * Adds the given selection to the list of selections.
     *
     * @param selection the selection object to add.
     */
    public void add(Selection selection)
    {
        selections.put(selection.getName(), selection);

        SelectionEvent selectionEvent = new SelectionEvent(selection);
        selectionEvent.setAdded(true);

        for(Iterator it = listeners.iterator(); it.hasNext();)
        {
            ((SelectionListener) it.next()).selectionListChanged(selectionEvent);
        }
    }

    /**
     * Adds the given selection listener to the list of listeners.
     *
     * @param listener the selection listener to add.
     */
    public void addSelectionListener(SelectionListener listener)
    {
        if(!listeners.contains(listener))
        {
            listeners.add(listener);
        }
    }

    /**
     * Removes the given selection for the list of selections.
     *
     * @param selection the selection to remove from the list.
     */
    public void remove(Selection selection)
    {
        selections.remove(selection.getName());

        SelectionEvent selectionEvent = new SelectionEvent(selection);
        selectionEvent.setAdded(false);

        for(Iterator it = listeners.iterator(); it.hasNext();)
        {
            ((SelectionListener) it.next()).selectionListChanged(selectionEvent);
        }
    }

    /**
     * Removes the given selection listener from the list of listeners.
     *
     * @param listener the selection listener to remove.
     */
    public void removeSelectionListener(SelectionListener listener)
    {
        listeners.remove(listener);
    }

    /**
     * Informs the registered listeners that the active session has changed.
     */
    public void selectionChanged()
    {
        Selection activeSel = (Selection) selections.get(activeSelection);
        SelectionEvent selectionEvent = new SelectionEvent(activeSel);

        for(Iterator it = listeners.iterator(); it.hasNext();)
        {
            ((SelectionListener) it.next()).selectionChanged(selectionEvent);
        }

        activeSel.committedChanges();
    }

    /**
     * DOCUMENT ME!
     */
    public void updateLastActive()
    {
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
